package org.jtheque.core.managers.module.beans;

import org.jtheque.core.managers.module.annotations.ModuleDefinition;
import org.jtheque.core.managers.module.annotations.Module;
import org.jtheque.core.managers.Managers;

import java.io.File;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @author Baptiste Wicht
 */
public final class ModuleContainer {
    private Object module;
    private final String beanName;

    private final Module infos;
    private final ModuleDefinition definition;
    private final boolean primary;

    private BeanMethod plugMethod;
    private BeanMethod prePlugMethod;
    private BeanMethod unPlugMethod;

    private ModuleState state;

    /**
     * Construct a new ModuleContainer. 
     * 
     * @param beanName The name of the bean. 
     * @param infos The informations about the module. 
     * @param definition The definition of the module. 
     * @param primary A boolean flag indicating if the module is a primary module or not. 
     */
    public ModuleContainer(String beanName, Module infos, ModuleDefinition definition, boolean primary) {
        super();
        
        this.beanName = beanName;
        this.infos = infos;
        this.definition = definition;
        this.primary = primary;
    }

    /**
     * Return the real module. 
     * 
     * @return The real module. 
     */
    public Object getModule() {
        return module;
    }

    /**
     * Return the plug method. 
     * 
     * @return The plug method. 
     */
    public BeanMethod getPlugMethod() {
        return plugMethod;
    }

    /**
     * Set the plug method. 
     * 
     * @param plugMethod The plug method. 
     */
    public void setPlugMethod(BeanMethod plugMethod) {
        this.plugMethod = plugMethod;
    }

    /**
     * Return the preplug method. 
     * 
     * @return The plug method. 
     */
    public BeanMethod getPrePlugMethod() {
        return prePlugMethod;
    }

    /**
     * Set the preplug method. 
     * 
     * @param prePlugMethod The preplug method. 
     */
    public void setPrePlugMethod(BeanMethod prePlugMethod) {
        this.prePlugMethod = prePlugMethod;
    }

    /**
     * Return the unplug method. 
     * 
     * @return The plug method. 
     */
    public BeanMethod getUnPlugMethod() {
        return unPlugMethod;
    }

    /**
     * Set the unplug method. 
     * 
     * @param unPlugMethod The unplug method. 
     */
    public void setUnPlugMethod(BeanMethod unPlugMethod) {
        this.unPlugMethod = unPlugMethod;
    }

    /**
     * Return the current state of the module. 
     * 
     * @return The current state of the module. 
     */
    public ModuleState getState() {
        return state;
    }

    /**
     * Set the state of the module. 
     * 
     * @param state The state of the module. 
     */
    public void setState(ModuleState state) {
        this.state = state;
    }

    /**
     * Return the infos of the module. 
     * 
     * @return The infos of the module. 
     */
    public Module getInfos() {
        return infos;
    }

    /**
     * Return the module definition. 
     * 
     * @return the module definition. 
     */
    public ModuleDefinition getModuleDefinition() {
        return definition;
    }

    /**
     * Indicate if the module has a module definition. 
     * 
     * @return true if the module has a module definition else false. 
     */
    public boolean hasModuleDefinition() {
        return definition != null;
    }

    /**
     * Indicate if the module is a primary one or not. 
     * 
     * @return true if the module is a primary one else false. 
     */
    public boolean isPrimaryModule() {
        return primary;
    }

    /**
     * Return the file of the module. 
     * 
     * @return The file of the module. 
     */
    public File getModuleFile(){
        return new File(Managers.getCore().getFolders().getModulesFolder(), infos.jarFile());
    }

    /**
     * Return the name of the bean. 
     * 
     * @return The name of the bean. 
     */
    public String getBeanName() {
        return beanName;
    }

    @Override
    public String toString() {
        return infos.name();
    }

    /**
     * Set the module instance. 
     * 
     * @param module The module instance. 
     */
    public void setModule(Object module) {
        this.module = module;
    }
}