package org.jtheque.core.managers.update.versions;

import org.jtheque.utils.bean.Version;
import org.jtheque.utils.collections.CollectionUtils;
import org.jtheque.core.managers.module.beans.ModuleContainer;
import org.jtheque.core.managers.update.Updatable;
import org.jtheque.core.managers.view.IViewManager;
import org.jtheque.core.managers.view.edt.SimpleTask;
import org.jtheque.core.managers.error.IErrorManager;

import javax.swing.JLabel;
import javax.annotation.Resource;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @author Baptiste Wicht
 */
public final class VersionsLoader implements IVersionsLoader {
    private final Map<Object, VersionsFile> cache;

    private static final int CACHE_INITIAL_SIZE = 20;

    private final Executor versionsLoader;

    @Resource
    private IViewManager viewManager;

    @Resource
    private IErrorManager errorManager;

    /**
     * Construct a new VersionsLoader.  
     */
    public VersionsLoader() {
        super();

        versionsLoader = Executors.newFixedThreadPool(5);

        cache = new HashMap<Object, VersionsFile>(CACHE_INITIAL_SIZE);
    }

    @Override
    public Version getVersion(Object object) {
        if(isModule(object)){
            return new Version(((ModuleContainer)object).getInfos().version());
        } else if (isUpdatable(object)){
            return ((Updatable)object).getVersion();
        }

        return null;
    }

    /**
     * Test if the object is a module. 
     * 
     * @param object The object to test. 
     * 
     * @return true if the object is a module else false. 
     */
    private static boolean isModule(Object object) {
        return object instanceof ModuleContainer;
    }

    /**
     * Test if the object is an updatable. 
     * 
     * @param object The object to test. 
     * 
     * @return true if the object is an updatable else false. 
     */
    private static boolean isUpdatable(Object object) {
        return object instanceof Updatable;
    }

    @Override
    public List<Version> getVersions(Object object) {
        VersionsFile file = getVersionsFile(object);

        return file == null ? CollectionUtils.<Version>emptyList() : CollectionUtils.expand(file.getVersions(), new VersionExpander());
    }

    @Override
    public List<OnlineVersion> getOnlineVersions(Object object) {
        return getVersionsFile(object).getVersions();
    }

    /**
     * Return the version's file of the object.
     *
     * @param object The object to get the version's file.
     *
     * @return The version's file of the module.
     */
    private VersionsFile getVersionsFile(Object object) {
        if (!cache.containsKey(object)) {
            cache.put(object, new VersionsFileReader().read(object));

            if(!cache.containsKey(object)){
                errorManager.addInternationalizedError("error.update.internet");
            }
        }

        return cache.get(object);
    }

    @Override
    public OnlineVersion getOnlineVersion(Version version, Object object) {
        for (OnlineVersion v : getOnlineVersions(object)) {
            if (v.getVersion().equals(version)) {
                return v;
            }
        }

        return null;
    }

    @Override
    public void displayVersionWhenLoaded(final Object object, final JLabel label) {
        versionsLoader.execute(new Runnable() {
            @Override
            public void run() {
                viewManager.execute(new RefreshLabelTask(label, object));
            }
        });
    }

    @Override
    public InstallVersion getInstallVersion(String versionFileURL) {
        return new VersionsFileReader().read(versionFileURL).getInstallVersion();
    }

    /**
     * A task to refresh the label.
     *
     * @author Baptiste Wicht
     */
    private final class RefreshLabelTask extends SimpleTask {
        private final JLabel label;
        private final Object object;

        /**
         * Construct a new RefreshLabelTask.
         *
         * @param label  The label to refresh.
         * @param object The object to manage.
         */
        private RefreshLabelTask(JLabel label, Object object) {
            super();

            this.label = label;
            this.object = object;
        }

        @Override
        public void run() {
            label.setText(getVersionsFile(object).getMostRecentVersion().getStringVersion());
            viewManager.refresh(label);
        }
    }
}
